/*
    uart programmer for Cypress PSoC.
    PSoC bootloader commands.
    Copyright (C) 2015  Michael Stickel (michael@cubic.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "psoc_uart_programmer_commands.h"
#include "psoc_uart_programmer_framer.h"
#include "tools.h"

#include <stdio.h>
#include <stdlib.h> // malloc, free
#include <string.h> // memcpy

enum PsocCommand
{
    PsocCommandVerifyChecksum = 0x31,
    PsocCommandGetFlashSize = 0x32,
    PsocCommandGetApplicationStatus = 0x33,
    PsocCommandEraseRow = 0x34,
    PsocCommandSyncBootloader = 0x35,
    PsocCommandSetActiveApplication = 0x36,
    PsocCommandSendData = 0x37,
    PsocCommandEnterBootloader = 0x38,
    PsocCommandProgramRow = 0x39,
    PsocCommandVerifyRow = 0x3A,
    PsocCommandExitBootloader = 0x3B
};

int PsocVerifyChecksum()
{
    unsigned char status = 0;
    int rxsize = 1;
    const int r = psoc_uartbootloader_transceive (PsocCommandExitBootloader, 0, 0, &status, &rxsize);
    if ((r != 0) && (rxsize > 0))
    {
	return (status == 0) ? 1 : 0;
	fprintf (stderr, "Error for ExitBootloader: status:%d\n", r);
    }
    return -1;
}

int PsocGetFlashSize(const int _flashArrayId, int * _firstRowNumber, int * _lastRowNumber)
{
    unsigned char rxbuf[4];
    int rxsize = sizeof(rxbuf);
    const unsigned char flashArrayId = _flashArrayId;
    const int r = psoc_uartbootloader_transceive (PsocCommandGetFlashSize, &flashArrayId, 1, rxbuf, &rxsize);
    if (r == 0)
    {
	if (rxsize == 4)
	{
	    *_firstRowNumber = rxbuf[0] | (rxbuf[1]<<8);
	    *_lastRowNumber = rxbuf[2] | (rxbuf[3]<<8);
	}
	else
	{
	    fprintf (stderr, "invalid result size for GetFlashSize\n");
	    return -1;
	}
    }
    else
    {
	fprintf (stderr, "Error for GetFlashSize: status:%d rxsize:%d\n", r, rxsize);
	return -2;
    }
    return 0;
}

int PsocGetApplicationStatus(const int _applicationNumber, int * _validApplicationNumber,  int * _activeApplicationNumber)
{
    unsigned char applicationNumber = _applicationNumber;
    unsigned char rxbuf[2];
    int rxsize = sizeof(rxbuf);
    const int r = psoc_uartbootloader_transceive (PsocCommandGetApplicationStatus, &applicationNumber, 1, rxbuf, &rxsize);
    if (r == 0)
    {
	if (_validApplicationNumber != 0)
	{
	    *_validApplicationNumber = rxbuf[0];
	}

	if (_activeApplicationNumber != 0)
	{
	    *_activeApplicationNumber = rxbuf[1];
	}
	return 0;
    }
    return 1;
}

int PsocEraseRow (const unsigned int _flashArrayId, const unsigned int _rowNumber)
{
    if ((_flashArrayId > 0xFF) || (_rowNumber > 0xFFFF))
    {
	return 1;
    }
    unsigned char txbuffer[3] = { (unsigned char)_flashArrayId, (unsigned char)_rowNumber, (unsigned char)(_rowNumber >> 8) };
    int dummy = 0;
    const int r = psoc_uartbootloader_transceive (PsocCommandEraseRow, txbuffer, sizeof(txbuffer), 0, &dummy);
    if (r == 0)
    {
	return 0;
    }
    return 1;
}

int PsocSyncBootloader()
{
    int dummy = 0;
    const int r = psoc_uartbootloader_transceive (PsocCommandSyncBootloader, 0, 0, 0, &dummy);
    if (r == 0)
    {
	return 0;
    }
    return 1;
}

int PsocSetActiveApplication(const unsigned int _applicationNumber)
{
    if (_applicationNumber > 0xFF)
    {
	return 1;
    }
    unsigned char applicationNumber = _applicationNumber;
    int dummy = 0;
    const int r = psoc_uartbootloader_transceive (PsocCommandSetActiveApplication, &applicationNumber, 1, 0, &dummy);
    if (r == 0)
    {
	return 0;
    }
    return 1;
}

int PsocSendData(const unsigned char * _data, const unsigned int _size)
{
    if (_size > 0xFFFF)
    {
	return 1;
    }
    int dummy = 0;
    const int r = psoc_uartbootloader_transceive (PsocCommandSendData, _data, _size, 0, &dummy);
    if (r == 0)
    {
	return 0;
    }
    return 1;
}

int PsocEnterBootloader (unsigned long * _siliconId, unsigned char * _siliconRev, int * _bootloaderVersion)
{
    unsigned char rxbuf[8];
    int rxsize = sizeof(rxbuf);
    const int r = psoc_uartbootloader_transceive (PsocCommandEnterBootloader, 0, 0, rxbuf, &rxsize);
    if (r == 0)
    {
	hexdump(rxbuf, rxsize);
	*_siliconId = rxbuf[0] | (rxbuf[1]<<8)  | (rxbuf[2]<<16)  | (rxbuf[3]<<24);
	*_siliconRev = rxbuf[4];
	*_bootloaderVersion = rxbuf[5] | (rxbuf[6]<<8)  | (rxbuf[7]<<16);
    }
    return r;
}

int PsocProgramRow(const unsigned int _flashArrayId, const unsigned int _rowNumber, const unsigned char * _data, const unsigned int _size)
{
    if ((_flashArrayId > 0xFF) || (_rowNumber > 0xFFF) || (_size > (0xFFFF-3)))
    {
	return 1;
    }

    unsigned char * txbuffer = (unsigned char *)malloc(3 + _size);
    if (txbuffer == 0)
    {
	return 1;
    }

    txbuffer[0] = (unsigned char)_flashArrayId;
    txbuffer[1] = (unsigned char)_rowNumber;
    txbuffer[2] = (unsigned char)(_rowNumber >> 8);
    memcpy (&txbuffer[3], _data, _size);

    int dummy = 0;
    const int r = psoc_uartbootloader_transceive (PsocCommandProgramRow, txbuffer, 3 + _size, 0, &dummy);

    free(txbuffer);

    if (r == 0)
    {
	return 0;
    }
    return 1;
}

int PsocVerifyRow(const unsigned int _flashArrayId, const unsigned int _rowNumber, unsigned int * _checksum)
{
    if ((_flashArrayId > 0xFF) || (_rowNumber > 0xFFF))
    {
	return 1;
    }
    unsigned char txbuffer[3] = { (unsigned char)_flashArrayId, (unsigned char)_rowNumber, (unsigned char)(_rowNumber >> 8) };
    unsigned char checksum = 0;
    int rxsize = 1;
    const int r = psoc_uartbootloader_transceive (PsocCommandVerifyRow, txbuffer, sizeof(txbuffer), &checksum, &rxsize);

    if ((r == 0) && (rxsize >= 1))
    {
	if (_checksum)
	{
	    *_checksum = checksum;
	}
	return 0;
    }
    return 1;
}

void PsocExitBootloader(void)
{
    const int r = psoc_uartbootloader_transceive (PsocCommandExitBootloader, 0, 0, 0, 0);
    if (r != 0)
    {
	fprintf (stderr, "Error for ExitBootloader: status:%d\n", r);
    }
}

int PsocOpenPort(const char * _deviceFilename)
{
    return psoc_uartbootloader_open_port(_deviceFilename);
}
