/*
    uart programmer for Cypress PSoC.
    transceive frames to and receive frames from the serial bootloader.
    Copyright (C) 2015  Michael Stickel (michael@cubic.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "psoc_uart_programmer_framer.h"
#include "psoc_uart_programmer_io.h"

#include <stdio.h>

static unsigned short calculate_psoc_checksum(unsigned short _checksum, const unsigned char * _txbuf, const int _txsize)
{
    int i;
    for (i = 0; i < _txsize; ++i)
    {
	_checksum += _txbuf[i];
    }
    return _checksum;
}

/*
 * transceive:
 * _cmd: 
 * _txbuf: 
 * _txsize: 
 * _rxbuf: 
 * _maxrxsize: 
 *
 * returns the status of the received datagram (positive numbers) or one of the following error codes:
 * -1 general receive error.
 * -2 rx checksum error
 */
int psoc_uartbootloader_transceive (const int _cmd,
				    const unsigned char * _txbuf, const int _txsize,
				    unsigned char * _rxbuf, int * _rxsize)
{
    int err = 0;
    unsigned char txheader[4] = { 1, (unsigned char)_cmd, (unsigned char)_txsize, (unsigned char)(_txsize>>8) };
    const unsigned short txchecksum = 0x10000 - calculate_psoc_checksum(calculate_psoc_checksum(0, txheader, 4), _txbuf, _txsize);
    unsigned char footer[3] = {(unsigned char)txchecksum, (unsigned char)(txchecksum>>8), 0x17};

    if ((_txbuf==0) && (_txsize > 0))
    {
	return -1;
    }

    if (psoc_write(txheader, 4) < 0)
    {
	return -10;
    }
    if (_txsize > 0)
    {
	if (_txbuf)
	{
	    if (psoc_write(_txbuf, _txsize) < 0)
	    {
		return -10;
	    }
	}
    }
    if (psoc_write(footer, 3) < 0)
    {
	return -10;
    }

    if (_rxsize)
    {
	{
	    unsigned char c;
	    int timeout = 10;
	    do
	    {
		if (psoc_read(&c, 1) < 0)
		{
		    return -20;
		}
	    } while ((c != 1) && (--timeout > 0));
#ifdef DEBUG
	    fprintf (stderr, "timeout:%d\n", timeout);
#endif
	}

	unsigned char rxheader[4];
	rxheader[0] = 1;
	if (psoc_read(&rxheader[1], 3) < 0)
	{
	    return -21;
	}

	unsigned short calculatedRxChecksum = calculate_psoc_checksum(0, rxheader, 4);
	unsigned short rxsize = rxheader[2] | (rxheader[3]<<8);

	if (rxsize > 0)
	{
	    if (*_rxsize < rxsize)
	    {
		printf ("%d < %d\n", *_rxsize, rxsize);
		// message payload does not fit into buffer
		err = -5;
		int i;
		for (i = 0; i < rxsize; ++i)
		{
		    unsigned char c;
		    if (psoc_read(&c, 1) < 0)
		    {
			return -22;
		    }
		}
	    }
	    else if (_rxbuf)
	    {
		// message payload fits.
		if (psoc_read(_rxbuf, rxsize) < 0)
		{
		    return -23;
		}
		calculatedRxChecksum = calculate_psoc_checksum(calculatedRxChecksum, _rxbuf, rxsize);
	    }
	}
	unsigned char rxfooter[3];
	if (psoc_read(rxfooter, 3) < 0)
	{
	    return -24;
	}
	if (err == 0)
	{
#ifdef DEBUG
hexdump(rxheader, 4);
if (_rxbuf && rxsize>0)
{
    hexdump(_rxbuf, rxsize);
}
hexdump(rxfooter, 3);
#endif
	    const unsigned short rxchecksum = rxfooter[0] | (rxfooter[1]<<8);
	    calculatedRxChecksum = 0x10000 - calculatedRxChecksum;
	    if (rxchecksum != calculatedRxChecksum)
	    {
		fprintf (stderr, "rxchecksum mismatch: %04X != %04X\n", rxchecksum, calculatedRxChecksum);
		err = -2;
	    }
	}
	if ((err==0) && (rxfooter[2] != 0x17))
	{
	  err = -3;
	}
	if ((err==0) && (rxheader[0] != 1))
	{
	  err = -4;
	}

	if (err == 0)
	  {
	    *_rxsize = rxsize;
	    err = rxheader[1];
	  }
    }
    return err;
}


int psoc_uartbootloader_open_port(const char * _deviceFilename)
{
    return open_psoc(_deviceFilename);
}



